@@ -1,17 +1,19 @@
 use anyhow::{format_err, Error};
 use cargo::{
-    core::manifest::ManifestMetadata,
-    core::registry::PackageRegistry,
-    core::source::MaybePackage,
     core::{
-        resolver::features::CliFeatures, Dependency, EitherManifest, FeatureValue, Manifest,
-        Package, PackageId, Registry, Source, SourceId, Summary, Target, TargetKind, Workspace,
+        manifest::ManifestMetadata, registry::PackageRegistry, resolver::features::CliFeatures,
+        Dependency, EitherManifest, FeatureValue, Manifest, Package, PackageId, Registry, SourceId,
+        Summary, Target, TargetKind, Workspace,
+    },
+    ops::{self, PackageOpts, Packages},
+    sources::{
+        source::{MaybePackage, QueryKind, Source},
+        IndexSummary, RegistrySource,
+    },
+    util::{
+        cache_lock::CacheLockMode, interning::InternedString, toml::read_manifest, FileLock,
+        GlobalContext,
     },
-    ops,
-    ops::{PackageOpts, Packages},
-    sources::RegistrySource,
-    util::{interning::InternedString, toml::read_manifest, FileLock},
-    Config,
 };
 use filetime::{set_file_times, FileTime};
 use flate2::read::GzDecoder;
@@ -21,12 +23,12 @@ use semver::Version;
 use tar::Archive;
 use tempfile;
 
-use std;
 use std::collections::{BTreeMap, HashSet};
 use std::fs;
 use std::hash::{Hash, Hasher};
-use std::io::{self, Read, Write};
+use std::io::{self, Read};
 use std::path::Path;
+use std::{self, ffi::OsStr};
 
 use crate::config::testing_ignore_debpolv;
 use crate::errors::*;
@@ -37,7 +39,7 @@ pub struct CrateInfo {
     // allows overriding package.manifest() e.g. via patches
     manifest: Manifest,
     crate_file: FileLock,
-    config: Config,
+    context: GlobalContext,
     source_id: SourceId,
     excludes: Vec<Pattern>,
     includes: Vec<Pattern>,
@@ -59,8 +61,8 @@ fn hash<H: Hash>(hashable: &H) -> u64 {
     hasher.finish()
 }
 
-fn fetch_candidates(registry: &mut PackageRegistry, dep: &Dependency) -> Result<Vec<Summary>> {
-    let mut summaries = match registry.query_vec(dep, false) {
+fn fetch_candidates(registry: &mut PackageRegistry, dep: &Dependency) -> Result<Vec<IndexSummary>> {
+    let mut summaries = match registry.query_vec(dep, QueryKind::Exact) {
         std::task::Poll::Ready(res) => res?,
         std::task::Poll::Pending => {
             registry.block_until_ready()?;
@@ -72,19 +74,19 @@ fn fetch_candidates(registry: &mut PackageRegistry, dep: &Dependency) -> Result<
 }
 
 pub fn invalidate_crates_io_cache() -> Result<()> {
-    let config = Config::default()?;
-    let _lock = config.acquire_package_cache_lock()?;
-    let source_id = SourceId::crates_io(&config)?;
+    let context = GlobalContext::default()?;
+    let _lock = context.acquire_package_cache_lock(CacheLockMode::DownloadExclusive)?;
+    let source_id = SourceId::crates_io_maybe_sparse_http(&context)?;
     let yanked_whitelist = HashSet::new();
-    let mut r = RegistrySource::remote(source_id, &yanked_whitelist, &config)?;
+    let mut r = RegistrySource::remote(source_id, &yanked_whitelist, &context)?;
     r.invalidate_cache();
     Ok(())
 }
 
 pub fn crate_name_ver_to_dep(crate_name: &str, version: Option<&str>) -> Result<Dependency> {
     // note: this forces a network call
-    let config = Config::default()?;
-    let source_id = SourceId::crates_io(&config)?;
+    let context = GlobalContext::default()?;
+    let source_id = SourceId::crates_io_maybe_sparse_http(&context)?;
     let version = version.and_then(|v| {
         if v.is_empty() {
             None
@@ -111,21 +113,23 @@ impl CrateInfo {
         version: Option<&str>,
         crate_path: &Path,
     ) -> Result<CrateInfo> {
-        let config = Config::default()?;
+        let context = GlobalContext::default()?;
         let crate_path = crate_path.canonicalize()?;
         let source_id = SourceId::for_path(&crate_path)?;
 
         let (package, crate_file) = {
             let yanked_whitelist = HashSet::new();
 
-            let mut source = source_id.load(&config, &yanked_whitelist)?;
+            let mut source = source_id.load(&context, &yanked_whitelist)?;
 
             let package_id = match version {
-                None | Some("") => {
+                None => {
                     let dep = Dependency::parse(crate_name, None, source_id)?;
                     let mut package_id: Option<PackageId> = None;
                     loop {
-                        match source.query(&dep, &mut |p| package_id = Some(p.package_id())) {
+                        match source.query(&dep, QueryKind::Exact, &mut |p| {
+                            package_id = Some(p.package_id())
+                        }) {
                             std::task::Poll::Ready(res) => {
                                 res?;
                                 break;
@@ -137,7 +141,10 @@ impl CrateInfo {
                     }
                     package_id.unwrap()
                 }
-                Some(version) => PackageId::new(crate_name, version, source_id)?,
+                Some(version) => {
+                    let version = version.parse::<Version>()?;
+                    PackageId::new(crate_name.into(), version, source_id)
+                }
             };
 
             let maybe_package = source.download(package_id)?;
@@ -151,10 +158,10 @@ impl CrateInfo {
             }?;
 
             let crate_file = {
-                let workspace = Workspace::ephemeral(package.clone(), &config, None, true)?;
+                let workspace = Workspace::ephemeral(package.clone(), &context, None, true)?;
 
                 let opts = PackageOpts {
-                    config: &config,
+                    gctx: &context,
                     verify: false,
                     list: false,
                     check_metadata: true,
@@ -179,7 +186,7 @@ impl CrateInfo {
                 workspace
                     .target_dir()
                     .join("package")
-                    .open_rw(filename, &config, "crate file")?
+                    .open_rw_exclusive_create(filename, &context, "crate file")?
             };
 
             (package, crate_file)
@@ -191,7 +198,7 @@ impl CrateInfo {
             package,
             manifest,
             crate_file,
-            config,
+            context,
             source_id,
             excludes: vec![],
             includes: vec![],
@@ -208,32 +215,32 @@ impl CrateInfo {
     }
 
     pub fn new_from_dependency(dependency: &Dependency, update: bool) -> Result<CrateInfo> {
-        let mut config = Config::default()?;
+        let mut context = GlobalContext::default()?;
         if !update {
             // unfriendly API from cargo; we'll have to make do with it for
             // now as there is no other alternative
-            config.configure(
+            context.configure(
                 0,
                 false,
                 None,
-                config.frozen(),
-                config.locked(),
+                context.frozen(),
+                context.locked(),
                 true, // offline
-                &config.target_dir()?.map(|x| x.into_path_unlocked()),
+                &context.target_dir()?.map(|x| x.into_path_unlocked()),
                 &[],
                 &[],
             )?;
         }
 
-        let source_id = dependency.source_id();
+        let source_id = SourceId::crates_io_maybe_sparse_http(&context)?;
         let registry_name = format!(
             "{}-{:016x}",
             source_id.url().host_str().unwrap_or(""),
             hash(&source_id).swap_bytes()
         );
-        let get_package_info = |config: &Config| -> Result<_> {
-            let lock = config.acquire_package_cache_lock()?;
-            let mut registry = PackageRegistry::new(config)?;
+        let get_package_info = |context: &GlobalContext| -> Result<_> {
+            let lock = context.acquire_package_cache_lock(CacheLockMode::DownloadExclusive)?;
+            let mut registry = PackageRegistry::new(context)?;
             registry.lock_patches();
             let summaries = fetch_candidates(&mut registry, dependency)?;
             drop(lock);
@@ -271,22 +278,22 @@ impl CrateInfo {
                 }
             }
             let filename = format!("{}-{}.crate", pkgid.name(), pkgid.version());
-            let crate_file = config
+            let crate_file = context
                 .registry_cache_path()
                 .join(&registry_name)
-                .open_ro(&filename, config, &filename)?;
+                .open_ro_shared(&filename, context, &filename)?;
             Ok((package.clone(), manifest.clone(), crate_file))
         };
         // if update is false but the user never downloaded the crate then the
         // first call will error; re-try with online in that case
         let (package, manifest, crate_file) =
-            get_package_info(&config).or_else(|_| get_package_info(&Config::default()?))?;
+            get_package_info(&context).or_else(|_| get_package_info(&GlobalContext::default()?))?;
 
         Ok(CrateInfo {
             package,
             manifest,
             crate_file,
-            config,
+            context,
             source_id,
             excludes: vec![],
             includes: vec![],
@@ -315,7 +322,7 @@ impl CrateInfo {
     }
 
     pub fn replace_manifest(&mut self, path: &Path) -> Result<&Self> {
-        if let (EitherManifest::Real(v), _) = read_manifest(path, self.source_id, &self.config)? {
+        if let EitherManifest::Real(v) = read_manifest(path, self.source_id, &self.context)? {
             self.manifest = v;
         }
         Ok(self)
@@ -329,8 +336,8 @@ impl CrateInfo {
         self.package.manifest_path()
     }
 
-    pub fn rust_version(&self) -> Option<&str> {
-        self.manifest.rust_version()
+    pub fn rust_version(&self) -> Option<String> {
+        self.manifest.rust_version().map(|v| v.to_string())
     }
 
     pub fn targets(&self) -> &[Target] {
@@ -667,8 +674,6 @@ impl CrateInfo {
         // Ensure that Cargo.toml is in standard form, e.g. does not contain
         // path dependencies, so can be built standalone (see #4030).
         let toml_path = path.join("Cargo.toml");
-        let ws = Workspace::new(&toml_path.canonicalize()?, &self.config)?;
-        let registry_toml = self.package.to_registry_toml(&ws)?;
         let mut actual_toml = String::new();
         fs::File::open(&toml_path)?.read_to_string(&mut actual_toml)?;
 
@@ -679,16 +684,35 @@ impl CrateInfo {
             // to handle it specially.
             let old_toml_path = path.join("Cargo.toml.orig");
             fs::copy(&toml_path, &old_toml_path)?;
-            fs::OpenOptions::new()
-                .write(true)
-                .truncate(true)
-                .open(&toml_path)?
-                .write_all(registry_toml.as_bytes())?;
-            debcargo_info!(
-                "Rewrote {:?} to canonical form\nOld backed up as {:?}",
-                &toml_path,
-                &old_toml_path,
-            );
+            let ws = Workspace::new(&toml_path.canonicalize()?, &self.context)?;
+            let opts = PackageOpts {
+                gctx: &self.context,
+                list: false,
+                check_metadata: false,
+                allow_dirty: true,
+                verify: false,
+                jobs: None,
+                keep_going: false,
+                to_package: cargo::ops::Packages::Default,
+                targets: Vec::new(),
+                cli_features: CliFeatures::new_all(true),
+            };
+            let res = cargo::ops::package_one(&ws, &self.package, &opts)?
+                .ok_or_else(|| format_err!("Packaging non-canonicalized crate failed!"))?;
+            let mut archive = Archive::new(GzDecoder::new(res.file()));
+            for entry in archive.entries()? {
+                let mut entry = entry?;
+                let entry_path = entry.path()?;
+                eprintln!("{entry_path:?}");
+                let components = entry_path.iter();
+                if components.clone().count() == 2
+                    && components.last() == Some(OsStr::new("Cargo.toml"))
+                {
+                    entry.unpack(&toml_path)?;
+                    break;
+                }
+            }
+            std::fs::remove_dir_all(path.join("target"))?;
             source_modified = true;
             // avoid lintian errors about package-contains-ancient-file
             // TODO: do we want to do this for unmodified tarballs? it would
