@@ -1,6 +1,7 @@
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
 use std::fs;
 use std::io::{self, ErrorKind, Read, Seek, Write as IoWrite};
+#[cfg(unix)]
 use std::os::unix::fs::PermissionsExt;
 use std::path::{Path, PathBuf};
 use std::process::Command;
@@ -308,12 +309,6 @@ pub fn prepare_debian_folder(
         )?;
     }
 
-    // debian/compat
-    {
-        let mut compat = file("compat")?;
-        writeln!(compat, "12")?;
-    }
-
     // debian/copyright
     {
         let mut copyright = io::BufWriter::new(file("copyright")?);
@@ -358,7 +353,7 @@ pub fn prepare_debian_folder(
                 )?;
                 writeln!(
                     watch,
-                    r"uversionmangle=s/(\d)[_\.\-\+]?((RC|rc|pre|dev|beta|alpha)\d*)$/$1~$2/ \"
+                    r"uversionmangle=s/(\d)[_\.\-\+]?((RC|rc|pre|dev|beta|alpha)\.?\d*)$/$1~$2/ \"
                 )?;
                 writeln!(
                     watch,
@@ -386,6 +381,7 @@ pub fn prepare_debian_folder(
     if testing_ignore_debpolv() {
         fs::create_dir_all(tempdir.path().join("debcargo_testing_bin"))?;
         let mut env_hack = file("debcargo_testing_bin/env")?;
+        #[cfg(unix)]
         env_hack.set_permissions(fs::Permissions::from_mode(0o777))?;
         // intercept calls to dh-cargo-built-using
         writeln!(
@@ -401,6 +397,7 @@ echo "debcargo testing: suppressing dh-cargo-built-using";;
     // debian/rules
     {
         let mut rules = file("rules")?;
+        #[cfg(unix)]
         rules.set_permissions(fs::Permissions::from_mode(0o777))?;
         if has_dev_depends || testing_ignore_debpolv() {
             // don't run any tests, we don't want extra B-D on dev-depends
@@ -615,6 +612,7 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
 
     let features_with_deps = crate_info.all_dependencies_and_features();
     let dev_depends = deb_deps(config, &crate_info.dev_dependencies())?;
+    let has_dev_deps = !dev_depends.is_empty();
     log::trace!(
         "features_with_deps: {:?}",
         features_with_deps
@@ -640,8 +638,9 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
         }
     };
 
+    let rustc = rustc_dep(&crate_info.rust_version());
     let build_deps = {
-        let build_deps = ["debhelper (>= 12)", "dh-cargo (>= 25)"]
+        let build_deps = ["debhelper-compat (= 13)", "dh-sequence-cargo"]
             .iter()
             .map(|x| x.to_string());
         // note: please keep this in sync with build_order::dep_features
@@ -653,11 +652,14 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
             PackageKey::feature("default"),
             &default_features,
         );
-        let rustc = rustc_dep(&crate_info.rust_version());
-        let build_deps_extra = ["cargo:native".into(), rustc, "libstd-rust-dev".into()]
-            .into_iter()
-            .chain(deb_deps(config, &default_deps)?)
-            .chain(extra_override_deps);
+        let build_deps_extra = [
+            "cargo:native".into(),
+            rustc.clone(),
+            "libstd-rust-dev".into(),
+        ]
+        .into_iter()
+        .chain(deb_deps(config, &default_deps)?)
+        .chain(extra_override_deps);
         if !bins.is_empty() {
             build_deps.chain(build_deps_extra).collect()
         } else {
@@ -667,15 +669,12 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
                 .collect()
         }
     };
+    let test_deps: Vec<String> = Some(rustc).into_iter().chain(dev_depends).collect();
     let mut source = Source::new(
         base_pkgname,
         name_suffix,
         crate_name,
-        if let Some(ref home) = meta.homepage {
-            home
-        } else {
-            ""
-        },
+        meta.homepage.as_deref().unwrap_or(""),
         lib,
         maintainer.to_string(),
         uploaders.iter().map(|s| s.to_string()).collect(),
@@ -707,24 +706,15 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
 
     if lib {
         // debian/tests/control
+        let all_features: Vec<&str> = features_with_deps.keys().copied().collect();
         let all_features_test_broken = match test_is_marked_broken("@") {
             Some(v) => v,
-            None => features_with_deps
-                .keys()
+            None => all_features
+                .iter()
                 .any(|f| test_is_marked_broken(f).unwrap_or(false)),
         };
-        let all_features_test_depends = Some(&"@")
-            .into_iter()
-            .chain(features_with_deps.keys())
-            .flat_map(|f| {
-                config
-                    .package_test_depends(PackageKey::feature(f))
-                    .into_iter()
-                    .flatten()
-            })
-            .map(|s| s.to_string())
-            .chain(dev_depends.clone())
-            .collect::<Vec<_>>();
+        let all_features_test_depends =
+            generate_test_dependencies("@", &all_features, config, &test_deps);
         let mut testctl = io::BufWriter::new(file("tests/control")?);
         write!(
             testctl,
@@ -811,62 +801,6 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
                 .collect::<Vec<_>>()
         );
         let (mut provides, reduced_features_with_deps) = if config.collapse_features {
-            debcargo_warn!(
-                "You are using the collapse_features work-around, which makes the resulting"
-            );
-            debcargo_warn!(
-                "package uninstallable when (now or in the future) your crate dependencies"
-            );
-            debcargo_warn!(
-                "contain cyclic dependencies on the crate-level; this is because cargo only"
-            );
-            debcargo_warn!("enforces acyclicity of dependencies on the per-feature level.");
-            debcargo_warn!("");
-            debcargo_warn!(
-                "By switching on collapse_features, you are telling debcargo to generate Debian"
-            );
-            debcargo_warn!(
-                "binary package on a per-crate-level basis and not a per-feature-level, meaning"
-            );
-            debcargo_warn!(
-                "that there is the chance of generating a dependency cycle on the Debian binary"
-            );
-            debcargo_warn!("package level, which APT by default refuses to install.");
-            debcargo_warn!("");
-            debcargo_warn!(
-                "You should not be doing this just because \"somebody told you so\"; you should"
-            );
-            debcargo_warn!(
-                "understand the situation and be prepared to deal with future technical debt"
-            );
-            debcargo_warn!("when the aforementioned cycles arise.");
-            debcargo_warn!("");
-            debcargo_warn!(
-                "Note that a long-term solution has been discussed with the FTP team and is in"
-            );
-            debcargo_warn!(
-                "progress - namely to move Debian rust packages into a separate section of the"
-            );
-            debcargo_warn!(
-                "archive, which will then have the stricter current NEW rules lifted, and then"
-            );
-            debcargo_warn!("the collapse_features work around would no longer be necessary.");
-            debcargo_warn!("");
-            debcargo_warn!("A basic example of the above would be:");
-            debcargo_warn!("");
-            debcargo_warn!("- crate A with feature AX depends on crate B with feature BY");
-            debcargo_warn!("- crate B with feature BX depends on crate A with feature AY");
-            debcargo_warn!("");
-            debcargo_warn!(
-                "This is a perfectly valid situation in the rust+cargo ecosystem. Notice that"
-            );
-            debcargo_warn!(
-                "there is no dependency cycle on the per-feature level, and this is enforced by"
-            );
-            debcargo_warn!(
-                "cargo; but if collapse_features is used then package A+AX+AY would cyclicly"
-            );
-            debcargo_warn!("depend on package B+BX+BY.");
             collapse_features(working_features_with_deps)
         } else {
             reduce_provides(working_features_with_deps)
@@ -1009,18 +943,7 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
                 }
 
                 // deps
-                let test_depends = Some(f)
-                    .into_iter()
-                    .chain(feature_deps)
-                    .flat_map(|f| {
-                        config
-                            .package_test_depends(PackageKey::feature(f))
-                            .into_iter()
-                            .flatten()
-                    })
-                    .map(|s| s.to_string())
-                    .chain(dev_depends.clone())
-                    .collect::<Vec<_>>();
+                let test_depends = generate_test_dependencies(f, &feature_deps, config, &test_deps);
                 let pkgtest = PkgTest::new(
                     package.name(),
                     crate_name,
@@ -1074,7 +997,27 @@ fn prepare_debian_control<F: FnMut(&str) -> std::result::Result<std::fs::File, s
         write!(control, "\n{}", bin_pkg)?;
     }
 
-    Ok((source, !dev_depends.is_empty(), test_is_broken("default")?))
+    Ok((source, has_dev_deps, test_is_broken("default")?))
+}
+
+fn generate_test_dependencies(
+    f: &str,
+    feature_deps: &[&str],
+    config: &Config,
+    test_deps: &[String],
+) -> Vec<String> {
+    Some(f)
+        .iter()
+        .chain(feature_deps)
+        .flat_map(|f| {
+            config
+                .package_test_depends(PackageKey::feature(f))
+                .into_iter()
+                .flatten()
+        })
+        .map(|s| s.to_string())
+        .chain(test_deps.to_owned())
+        .collect::<Vec<_>>()
 }
 
 fn collapse_features(
@@ -1172,7 +1115,7 @@ fn reduce_provides(
     (provides, features_with_deps)
 }
 
-fn rustc_dep(min_ver: &Option<&str>) -> String {
+fn rustc_dep(min_ver: &Option<String>) -> String {
     if let Some(min_ver) = min_ver {
         format!("rustc:native (>= {})", min_ver)
     } else {
@@ -1186,7 +1129,10 @@ mod test {
 
     #[test]
     fn rustc_dep_includes_minver() {
-        assert_eq!("rustc:native (>= 1.65)", rustc_dep(&Some(&"1.65")));
+        assert_eq!(
+            "rustc:native (>= 1.65)",
+            rustc_dep(&Some("1.65".to_string()))
+        );
     }
 
     #[test]
@@ -1200,6 +1146,7 @@ fn changelog_or_new(tempdir: &Path) -> Result<(fs::File, String)> {
         .read(true)
         .write(true)
         .create(true)
+        .truncate(false)
         .open(tempdir.join("changelog"))?;
     let mut changelog_data = String::new();
     changelog.read_to_string(&mut changelog_data)?;
