@@ -1,10 +1,13 @@
+use std::borrow::Cow;
 use std::collections::{BTreeMap, BTreeSet, VecDeque};
 use std::ffi::OsStr;
 use std::fmt;
 use std::fs;
 use std::io::{BufRead, BufReader, Error};
 use std::iter::Iterator;
+#[cfg(unix)]
 use std::os::unix::ffi::OsStrExt;
+#[cfg(unix)]
 use std::os::unix::fs::symlink;
 use std::path::{Path, PathBuf};
 use std::process::Command;
@@ -14,19 +17,35 @@ use walkdir::WalkDir;
 
 pub const HINT_SUFFIX: &str = ".debcargo.hint";
 
-pub fn hint_file_for(file: &Path) -> Option<&Path> {
+#[cfg(unix)]
+pub fn hint_file_for(file: &Path) -> Option<Cow<'_, Path>> {
     let file = file.as_os_str().as_bytes();
     if file.len() >= HINT_SUFFIX.len()
         && &file[file.len() - HINT_SUFFIX.len()..] == HINT_SUFFIX.as_bytes()
     {
-        Some(Path::new(OsStr::from_bytes(
+        Some(Cow::Borrowed(Path::new(OsStr::from_bytes(
             &file[..file.len() - HINT_SUFFIX.len()],
-        )))
+        ))))
     } else {
         None
     }
 }
 
+#[cfg(not(unix))]
+pub fn hint_file_for(file: &Path) -> Option<Cow<'_, Path>> {
+    if let Some(file_str) = file.to_str() {
+        if file_str.ends_with(HINT_SUFFIX) {
+            let trimmed_path = &file_str[..file_str.len() - HINT_SUFFIX.len()];
+            Some(Cow::Owned(PathBuf::from(trimmed_path)))
+        } else {
+            None
+        }
+    } else {
+        // Handle the case where the path is not representable as a string
+        None
+    }
+}
+
 pub fn lookup_fixmes(srcdir: &Path) -> Result<BTreeSet<PathBuf>, Error> {
     let mut fixmes = BTreeSet::new();
     for entry in WalkDir::new(srcdir) {
@@ -36,8 +55,8 @@ pub fn lookup_fixmes(srcdir: &Path) -> Result<BTreeSet<PathBuf>, Error> {
             let reader = BufReader::new(file);
             // If we find one FIXME we break the loop and check next file. Idea
             // is only to find files with FIXME strings in it.
-            for line in reader.lines().flatten() {
-                if line.contains("FIXME") {
+            for line in reader.lines() {
+                if line?.contains("FIXME") {
                     fixmes.insert(entry.path().to_path_buf());
                     break;
                 }
@@ -49,7 +68,7 @@ pub fn lookup_fixmes(srcdir: &Path) -> Result<BTreeSet<PathBuf>, Error> {
     let fixmes = fixmes
         .iter()
         .filter(|f| match hint_file_for(f) {
-            Some(ff) => fixmes.contains(ff) || !ff.exists(),
+            Some(ff) => fixmes.contains(ff.as_ref()) || !ff.exists(),
             None => true,
         })
         .cloned()
@@ -70,12 +89,18 @@ pub fn copy_tree(oldtree: &Path, newtree: &Path) -> Result<(), Error> {
         let oldpath = entry.path();
         let newpath = newtree.join(oldpath.strip_prefix(oldtree).unwrap());
         let ftype = entry.file_type();
-        if ftype.is_dir() {
-            fs::create_dir(newpath)?;
-        } else if ftype.is_file() {
-            fs::copy(oldpath, newpath)?;
-        } else if ftype.is_symlink() {
-            symlink(fs::read_link(oldpath)?, newpath)?;
+        match ftype {
+            f if f.is_dir() => {
+                fs::create_dir(newpath)?;
+            }
+            f if f.is_file() => {
+                fs::copy(oldpath, newpath)?;
+            }
+            #[cfg(unix)]
+            f if f.is_symlink() => {
+                symlink(fs::read_link(oldpath)?, newpath)?;
+            }
+            _ => {}
         }
     }
     Ok(())
